@@ -277,8 +277,8 @@ sub request
     # We ignore the kernel for now, but we should really use it to decide
     # weither we should run the request or not
     my $to=specifier_parse($request->{event});
-    eval
-    {
+
+    eval {
         die "$request->{event} isn't a valid specifier" unless $to;
         my $args=$request->{params};
         # allow proxied states to have multiple ARGs
@@ -313,8 +313,8 @@ sub request
         my $session=$kernel->alias_resolve($to->{session});
         die "Unknown session '$to->{session}'\n" unless $session;
 
-        _thunked_post($request->{rsvp},
-                      [$session, $to->{state}, @$args], $request->{from});
+        _thunked_post($request->{rsvp}, [$session, $to->{state}, @$args], 
+                      $request->{from}, $request->{wantarray});
     };
 
 
@@ -880,22 +880,29 @@ sub DEBUG { 0 }
 
 sub _start
 {
-    my($kernel, $heap, $name, $caller, $args, $foreign)=
-                @_[KERNEL, HEAP, ARG0, ARG1, ARG2, ARG3];
-    $heap->{rsvp}=$foreign;
+    my($kernel, $heap,          $name, $caller, $args, $rsvp, $wantarray)=
+                @_[KERNEL, HEAP, ARG0, ARG1,    ARG2,  ARG3,  ARG4];
+    $heap->{rsvp}=$rsvp;
     $heap->{name}=$name;
 
     DEBUG && print "$name created\n";
 
-    if($caller)                             # foreign session wants return
-    {
-        my $ret=$kernel->call(@$args);
-        if(defined $ret) {
-            DEBUG && print "Posted response '$ret' to ", Dumper $caller;
-            $kernel->post('IKC', 'post', $caller, $ret) ;
+    if($caller) {                           # foreign session wants return
+        
+        DEBUG and do { warn "Wants an array" if $wantarray};
+
+        my @ret;
+        @ret=$kernel->call(@$args) if $wantarray;
+        $ret[0]=$kernel->call(@$args) unless $wantarray;
+        if(defined @ret) {
+            local $"=', ';
+            DEBUG && print "Posted response '@ret' to ", Dumper $caller;
+            
+            $kernel->post('IKC', 'post', $caller, 
+                                ($wantarray ? \@ret : $ret[0]));
         }
-    } else
-    {
+    } 
+    else {
         $kernel->call(@$args);
     }
     return;
