@@ -27,10 +27,10 @@ require Exporter;
 
 @ISA = qw(Exporter);
 @EXPORT = qw(create_ikc_server);
-$VERSION = '0.10';
+$VERSION = '0.11';
 
 sub DEBUG { 0 }
-sub DEBUG_USR2 { 0 }
+sub DEBUG_USR2 { 1 }
 
 ###############################################################################
 #----------------------------------------------------
@@ -48,7 +48,7 @@ sub create_ikc_server
                         _start _stop error 
                         accept fork retry waste_time 
                         babysit rogues
-                        sig_CHLD sig_INT sig_other
+                        sig_CHLD sig_INT sig_USR2
                         )],
                     [\%params],
                   );
@@ -108,7 +108,7 @@ sub _start
 
     $kernel->sig('CHLD', 'sig_CHLD');
     $kernel->sig('INT', 'sig_INT');
-    DEBUG_USR2 and $kernel->sig('USR2', 'sig_other');
+    DEBUG_USR2 and $kernel->sig('USR2', 'sig_USR2');
 
                                         # keep track of children
     $heap->{children} = {};
@@ -120,7 +120,8 @@ sub _start
     foreach (2..$params->{processes}) { # fork the initial set of children
         $kernel->yield('fork');
     }
-    $kernel->yield('waste_time');    
+
+    $kernel->delay('waste_time', 60);
     if($params->{babysit}) {
         $heap->{babysit}=$params->{babysit};
         delete($heap->{"proctable"});
@@ -143,6 +144,10 @@ sub waste_time
     my($kernel, $heap)=@_[KERNEL, HEAP];
     return if $heap->{'is a child'};
 
+    unless($heap->{'been told we are parent'}) {
+        $heap->{'been told we are parent'}=1;
+        $kernel->signal($kernel, '__parent');
+    }
     if($heap->{'die'}) {
         DEBUG and warn "$$: Orderly shutdown\n";
     } else {
@@ -201,7 +206,7 @@ sub babysit
                     $ok{$pid}=1;
                 }
 
-            } elsif($state eq 'sleep') {
+            } elsif($state eq 'sleep' or $state eq 'defunct') {
                 $ok{$pid}=1;
                 # do nothing
             } else {
@@ -309,7 +314,8 @@ eval {
 
 sub _stop 
 {
-    my $heap = $_[HEAP];
+    my($kernel, $heap) = @_[KERNEL, HEAP];
+    check_kernel($kernel, $heap->{'is a child'});
                                         # kill the child servers
     if($heap->{children}) {
         foreach (keys %{$heap->{children}}) {
@@ -317,7 +323,7 @@ sub _stop
             kill 2, $_ or warn "$$: $_ $!\n";
         }
     }
-    DEBUG && print "$$: server is stopped\n";
+    DEBUG && print "$$: server is stoping\n";
 }
 
 #----------------------------------------------------
@@ -350,7 +356,7 @@ sub accept
                             " (Connection $heap->{connections})\n" : "\n");
     }
     if($heap->{children} and not $heap->{'is a child'}) {
-        warn "Parent process received a connection: THIS SUCKS\n";
+        warn "$$: Parent process received a connection: THIS SUCKS\n";
         _select_define($heap, 0);
         return;
     }
@@ -362,8 +368,13 @@ sub accept
     if ($heap->{'is a child'}) {
 
         if (--$heap->{connections} < 1) {
+            DEBUG and warn "$$: Game over\n";
+            $kernel->delay('waste_time');
             delete $heap->{wheel};
-            $kernel->yield('_stop');
+            check_kernel($kernel);
+#            $kernel->yield('_stop');
+        } else {
+            DEBUG and warn "$$: $heap->{connections} left\n";
         }
     } else {
         warn "$$: Master client got a connect!  this sucks!\n";
@@ -385,7 +396,7 @@ sub fork
         ## warn "$$: We are a child, why are we forking?\n";
         return;
     }
-
+    my $parent=$$;
                                         # try to fork
     my $pid = fork();
                                         # did the fork fail?
@@ -413,17 +424,25 @@ sub fork
                                         # child becomes a child server
     else {
         $heap->{verbose} and warn "$$: Created ", scalar localtime, "\n";
+        # Clean out stuff that the parent needs but not the children
+
         $heap->{'is a child'}   = 1;        # don't allow fork
-        $heap->{'fail forks'}   = 0;
+        $heap->{'failed forks'} = 0;
         $heap->{children}={};               # don't kill child processes
                                             # limit sessions, then die off
         $heap->{connections}    = $heap->{"max connections"};   
 
+        $kernel->sig('CHLD');
+        $kernel->sig('INT');
+        $kernel->delay('babysit') if $heap->{'babysit'};
+        delete @{$heap}{qw(rogues proctable)};
+
+        # Tell everyone we are now a child
+        $kernel->signal($kernel, '__child');
+
         # Create a select for the children, so that SocketFactory can
         # do it's thing
         _select_define($heap, 1);
-        $kernel->sig('CHLD');
-        $kernel->sig('INT');
 
         DEBUG && print "$$: child server has been forked\n";
     }
@@ -508,12 +527,38 @@ sub sig_INT
     return 0;
 }
 
+sub check_kernel
+{
+    my($kernel, $child, $signal)=@_;
+    if(ref $kernel) {
+        # 3 = KR_STATES
+        # 8 = KR_ALARMS
+        # 2 = KR_HANDLES
+        # 14 = KR_EXTRA_REFS
+    
+        warn( "$$ ----- Kernel Activity -----\n",
+               "| States : ", scalar(@{$kernel->[3]}), "\n",
+               "| Alarms : ", scalar(@{$kernel->[8]}), "\n",
+               "| Files  : ", scalar(keys(%{$kernel->[2]})), "\n",
+               "| Extra  : ", $kernel->[14], "\n",
+               "$$ ---------------------------\n"
+             ) if $signal; 
+        if($child) {
+            foreach my $q (@{$kernel->[8]}) {
+                warn "************ Alarm for ", join '/', @{$q->[0][2]{$q->[2]}};
+            }
+        }
+    } else {
+        warn "$kernel isn't a reference";
+    }
+}
 
-sub sig_other
+sub sig_USR2
 {
 #    return unless DEBUG;
-    my ($signal, $pid) = @_[ARG0, ARG1];
+    my ($kernel, $heap, $signal, $pid) = @_[KERNEL, HEAP, ARG0, ARG1];
     $pid||='';
+    check_kernel($kernel, $heap->{'is a child'}, 1);
     warn "$$: signal $signal $pid\n";
 }
 
